@@ -359,10 +359,16 @@ class Frame(object):
     """A single frame in a traceback."""
 
     def __init__(self, exc_type, exc_value, tb, context=None):
-        self.lineno = tb.tb_lineno
-        self.function_name = tb.tb_frame.f_code.co_name
-        self.locals = tb.tb_frame.f_locals
-        self.globals = tb.tb_frame.f_globals
+        if inspect.isframe(tb):
+            self.lineno = tb.f_lineno
+            tb_frame = tb
+        else:
+            self.lineno = tb.tb_lineno
+            tb_frame = tb.tb_frame
+
+        self.function_name = tb_frame.f_code.co_name
+        self.locals = tb_frame.f_locals
+        self.globals = tb_frame.f_globals
         self.context = context
 
         fn = inspect.getsourcefile(tb) or inspect.getfile(tb)
@@ -374,7 +380,7 @@ class Frame(object):
         self.filename = fn
         self.module = self.globals.get('__name__')
         self.loader = self.globals.get('__loader__')
-        self.code = tb.tb_frame.f_code
+        self.code = tb_frame.f_code
 
         # support for paste's traceback extensions
         self.hide = self.locals.get('__traceback_hide__', False)
@@ -386,6 +392,21 @@ class Frame(object):
                 info = str(info).decode('utf-8', 'replace')
         self.info = info
 
+    def __getattr__(self, item):
+        # Make it look like a plain python frame/traceback object.
+        if item in ('tb_lineno', 'f_lineno'):
+            return self.lineno
+        elif item in ('f_locals', ):
+            return self.locals
+        elif item in ('tb_frame', ):
+            return self
+        elif item in ('f_globals', ):
+            return self.globals
+        elif item in ('f_code', ):
+            return self.code
+
+        raise AttributeError("'Frame' object has no attribute '%s'" % item)
+
     def render(self):
         """Render a single frame in a traceback."""
         return FRAME_HTML % {
